package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.GicMessageDTO;
import com.gic.haoban.manage.api.dto.QywxCallBackDTO;
import com.gic.haoban.manage.api.dto.StaffDepartmentRelatedDTO;
import com.gic.haoban.manage.api.enums.GicDataType;
import com.gic.haoban.manage.api.enums.GicEditType;
import com.gic.haoban.manage.api.enums.WxEditType;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.MessageApiService;
import com.gic.haoban.manage.service.entity.*;
import com.gic.haoban.manage.service.service.*;
import com.gic.mq.sdk.GicMQClient;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 接收企业微信或者gic推送消息进行处理
 * Created by tgs on 2020/2/20.
 */
@Service("messageApiService")
public class MessageApiServiceImpl implements MessageApiService {
    private static final Logger log = LogManager.getLogger(MessageApiServiceImpl.class);

    @Autowired
    private DepartmentService departmentService;
    @Autowired
    private WxEnterpriseRelatedService wxEnterpriseRelatedService;
    @Autowired
    private DepartmentApiService departmentApiService;
    @Autowired
    private WxEnterpriseService enterpriseService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private StaffDepartmentRelatedService staffDepartmentRelatedService;

    @Override
    public void gicMessageReceive(GicMessageDTO gicMessageDTO) {
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            log.info("发送消息到dealGicMessageMq，{}", JSON.toJSONString(gicMessageDTO));
            clientInstance.sendMessage("dealGicMessageMq", JSON.toJSONString(gicMessageDTO));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void wxMessageReceive(QywxCallBackDTO qywxCallBackDTO) {
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            log.info("发送消息到dealWxMessageMq，{}", JSON.toJSONString(qywxCallBackDTO));
            clientInstance.sendMessage("dealWxMessageMq", JSON.toJSONString(qywxCallBackDTO));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void dealGicMessage(String param) {
        GicMessageDTO dto = JSON.parseObject(param, GicMessageDTO.class);
        TabHaobanWxEnterpriseRelated enterprise =
                this.wxEnterpriseRelatedService.getEnterpriseByGicEnterpriseId(dto.getEnterpriseId());
        if(enterprise != null){
            DepartmentDTO departmentDTO = new DepartmentDTO();
            departmentDTO.setDepartmentName(dto.getName());
            departmentDTO.setIsStore(dto.getType() != GicDataType.STORE.getCode() ? 0 : 1);
            departmentDTO.setRelatedId(dto.getId());
            DepartmentDTO parentDepartment = this.getParentDepartment(dto.getParentChain(), dto.getType(), enterprise.getWxEnterpriseId());
            departmentDTO.setParentDepartmentId(parentDepartment.getDepartmentId());
            departmentDTO.setChainId(parentDepartment.getChainId()+ Constant.ID_SEPARATOR + parentDepartment.getDepartmentId());
            departmentDTO.setChainName(parentDepartment.getChainName() + Constant.NAME_SEPARATOR + parentDepartment.getChainName());
            departmentDTO.setLevel(parentDepartment.getLevel() + 1);
            departmentDTO.setWxEnterpriseId(enterprise.getWxEnterpriseId());
            if(dto.getType() == GicEditType.ADD.getCode()){
                HaobanResponse response = this.departmentApiService.add(departmentDTO);
                log.info("添加结果：{}", JSON.toJSONString(response));
            }else if(dto.getType() == GicEditType.UPDATE.getCode()){
                TabHaobanDepartment department = this.departmentService.selectByRelatedId(dto.getId());
                if(department == null){
                    HaobanResponse response = this.departmentApiService.add(departmentDTO);
                    log.info("添加结果：{}", JSON.toJSONString(response));
                }else {
                    departmentDTO.setDepartmentId(department.getDepartmentId());
                    this.departmentApiService.edit(departmentDTO);
                    log.info("gic同步更新完成");
                }
            }else {
                TabHaobanDepartment department = this.departmentService.selectByRelatedId(dto.getId());
                if(department != null){
                    this.departmentApiService.del(department.getDepartmentId());
                    log.info("gic同步删除完成");
                }else {
                    log.info("gic同步删除不存在");
                }
            }
        }

    }

    @Override
    public void dealWxMessage(String param) {
        QywxCallBackDTO dto = JSON.parseObject(param, QywxCallBackDTO.class);
        if(dto.getChangeType().endsWith("party")){
            this.dealDepartment(dto);
        }else {
            this.dealUser(dto);
        }
    }

    private void dealDepartment(QywxCallBackDTO qywxCallBackDTO){
        DepartmentDTO departmentDTO = new DepartmentDTO();
        departmentDTO.setDepartmentName(qywxCallBackDTO.getDepartmentName());
        departmentDTO.setIsStore(0);
        departmentDTO.setRelatedId(qywxCallBackDTO.getId()+"");
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        TabHaobanDepartment parentDepartment = this.departmentService.getByWxId(qywxCallBackDTO.getParentId() + "", enterprise.getWxEnterpriseId());
        if(parentDepartment == null){
            return;
        }
        departmentDTO.setParentDepartmentId(parentDepartment.getDepartmentId());
        departmentDTO.setChainId(parentDepartment.getChainId()+ Constant.ID_SEPARATOR + parentDepartment.getDepartmentId());
        departmentDTO.setChainName(parentDepartment.getChainName() + Constant.NAME_SEPARATOR + parentDepartment.getChainName());
        departmentDTO.setLevel(parentDepartment.getLevel() + 1);
        departmentDTO.setWxEnterpriseId(enterprise.getWxEnterpriseId());
        departmentDTO.setWxDepartmentId(qywxCallBackDTO.getId()+"");
        if(qywxCallBackDTO.getChangeType().equals(WxEditType.ADDDEPART.getCode())){
            String response = this.departmentService.add(departmentDTO);
            log.info("微信同步新增部门：{}", JSON.toJSONString(response));
        }else if(qywxCallBackDTO.getChangeType().equals(WxEditType.UPDATEDEPART.getCode())){
            TabHaobanDepartment department = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
            if(department == null){
                String response = this.departmentService.add(departmentDTO);
                log.info("微信同步新增部门：{}", JSON.toJSONString(response));
            }else {
                departmentDTO.setDepartmentId(department.getDepartmentId());
                this.departmentService.edit(departmentDTO);
                log.info("企业微信更新同步完成");
            }
        }else {
            TabHaobanDepartment department = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
            if(department == null){
                log.info("微信同步删除部门不存在");
            }else {
                this.departmentService.del(departmentDTO.getDepartmentId());
                log.info("企业微信删除同步完成");
            }
        }
    }

    private void dealUser(QywxCallBackDTO qywxCallBackDTO){
        TabHaobanStaff staff = new TabHaobanStaff();
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        if(enterprise != null){
            staff.setWxEnterpriseId(enterprise.getWxEnterpriseId());
            staff.setActiveFlag(0);
            staff.setPhoneNumber(qywxCallBackDTO.getMobile());
            staff.setNickName(qywxCallBackDTO.getAlias());
            staff.setSex(qywxCallBackDTO.getGender());
            staff.setStaffName(qywxCallBackDTO.getUserName());
            staff.setWxUserId(qywxCallBackDTO.getUserid());
            staff.setPostion(qywxCallBackDTO.getPosition());
            if(qywxCallBackDTO.getChangeType().equals(WxEditType.ADDUSER.getCode())){
                this.addUser(qywxCallBackDTO, staff);
            } else if(qywxCallBackDTO.getChangeType().equals(WxEditType.UPDATEUSER.getCode())){
                TabHaobanStaff oldStaff = this.staffService.selectByUserId(qywxCallBackDTO.getUserid());
                if(oldStaff != null){
                    staff.setWxUserId(qywxCallBackDTO.getNewUserid());
                    staff.setStaffId(oldStaff.getStaffId());
                    this.staffService.updateByPrimaryKey(staff);
                    this.staffDepartmentRelatedService.delByUserid(qywxCallBackDTO.getUserid());
                    this.addUserDepart(qywxCallBackDTO, oldStaff.getStaffId(), enterprise.getWxEnterpriseId());
                }else {
                    this.addUser(qywxCallBackDTO, staff);
                }
            } else {
                TabHaobanStaff oldStaff = this.staffService.selectByUserId(qywxCallBackDTO.getUserid());
                if(oldStaff != null){
                    this.staffService.delByuserid(oldStaff.getStaffId());
                }else {
                    log.info("微信同步删除的门店不存在，{}", qywxCallBackDTO.getUserid());
                }
            }
        }else {
            log.info("授权企业不存在，{}", qywxCallBackDTO.getAuthCorpId());
        }
    }


    private void addUser(QywxCallBackDTO qywxCallBackDTO, TabHaobanStaff staff){
        String add = this.staffService.add(staff);
        log.info("新增用户返回结果：{}", add);
        this.addUserDepart(qywxCallBackDTO, add, staff.getWxEnterpriseId());
    }

    private void addUserDepart(QywxCallBackDTO qywxCallBackDTO, String staffId, String wxEnterpriseId){
        String[] departArr = qywxCallBackDTO.getDepartment();
        if(departArr != null){
            for(String s : departArr){
                TabHaobanDepartment department = this.departmentService.getByWxId(s, wxEnterpriseId);
                if(department == null){
                    log.info("部门不存在");
                    continue;
                }
                StaffDepartmentRelatedDTO related = new StaffDepartmentRelatedDTO();
                related.setWxEnterpriseId(wxEnterpriseId);
                related.setWxUserId(qywxCallBackDTO.getUserid());
                related.setClerkCode(qywxCallBackDTO.getUserid());
                related.setDepartmentId(department.getDepartmentId());
                related.setPhoneNumber(qywxCallBackDTO.getMobile());
                //related.setStaffDepartmentRelatedId(staffId);
                related.setStaffId(staffId);
                related.setStatusFlag(1);
                String add1 = this.staffDepartmentRelatedService.add(related);
                log.info("新增用户部门关联关系：{}", add1);
            }
        }
    }
    /**
     * 通过关联的父级ID链查询父级部门
     * @param parentChain
     * @return
     */
    private DepartmentDTO getParentDepartment(String parentChain, int dataType, String wxEnterpriseId){
        if(StringUtils.isNotBlank(parentChain)){
            String[] arr = parentChain.split("_");
            int len = arr.length;
            int startIndex = len - 2;
            if(dataType == GicDataType.STORE.getCode()){
                startIndex = len - 1;
            }
            for(int i = startIndex; i< len; i--){
                DepartmentDTO department = this.departmentService.getByRelatedId(arr[i]);
                if(department != null){
                    return department;
                }
            }
        }
        DepartmentDTO departmentDTO = this.departmentService.getRootByEnterpriseId(wxEnterpriseId);
        return departmentDTO;
    }
}
